#!/usr/bin/env python
# -*- coding: utf-8 -*-

#
# This file is subject to the terms and conditions defined in
# file 'LICENSE.md', which is part of this source code package.
#

# from kubernetes.models.v1beta1.CPUTargetUtilization import CPUTargetUtilization
# from kubernetes.models.v1beta1.DaemonSet import DaemonSet
# from kubernetes.models.v1beta1.DaemonSetSpec import DaemonSetSpec
# from kubernetes.models.v1beta1.DaemonSetStatus import DaemonSetStatus
# from kubernetes.models.v1beta1.Deployment import Deployment
# from kubernetes.models.v1beta1.DeploymentRollback import DeploymentRollback
# from kubernetes.models.v1beta1.DeploymentSpec import DeploymentSpec
# from kubernetes.models.v1beta1.DeploymentStatus import DeploymentStatus
# from kubernetes.models.v1beta1.DeploymentStrategy import DeploymentStrategy
# from kubernetes.models.v1beta1.LabelSelector import LabelSelector
# from kubernetes.models.v1beta1.LabelSelectorRequirement import LabelSelectorRequirement
# from kubernetes.models.v1beta1.ReplicaSet import ReplicaSet
# from kubernetes.models.v1beta1.ReplicaSetSpec import ReplicaSetSpec
# from kubernetes.models.v1beta1.ReplicaSetStatus import ReplicaSetStatus
# from kubernetes.models.v1beta1.RollbackConfig import RollbackConfig
# from kubernetes.models.v1beta1.RollingUpdateDeployment import RollingUpdateDeployment
# from kubernetes.models.v1beta1.StatefulSet import StatefulSet
# from kubernetes.models.v1beta1.StatefulSetSpec import StatefulSetSpec
# from kubernetes.models.v1beta1.StatefulSetStatus import StatefulSetStatus
# from kubernetes.models.v1beta1.StorageClass import StorageClass
# from kubernetes.models.v1beta1.SubresourceReference import SubresourceReference
